#!/usr/bin/python

# Alarm scriopt for FTS
# G. McCance 2006
# Scans the FTA agent config directory for config files.
# and checks that the daemons defined there are running.
# (add an agent to the veto file to ignore it)
# The script will write an alarm file is an agent is down for more than 2 attempts.

import os
import sys
import fnmatch
import re
import string
import getopt
import commands
import syslog
import time
import shutil

def usage():
   print 'Usage:'
   print '\t', sys.argv[0], '[ --debug ] [ vetofile=filename ] [ statefile=filename ] [ alarmfile=filename ]' 

def do_restart(agent):
   print "Restarting agent: " + agent

def log_message(message):
   message = time.strftime("%a, %d %b %Y %H:%M:%S +0000", time.gmtime()) + ": " + message
   print message   

def main():
	debug=False
 
        if os.geteuid() != 0:
	   log_message('You must run this script as root!')
	   sys.exit(1)

        if not os.path.isdir('/opt/glite/etc/glite-data-transfer-agents.d'):
	   log_message('No FTA daemons configured on this node')
	   sys.exit(0)

	try:
   	   opts = getopt.getopt(sys.argv[1:], '', [ 'debug', 'vetofile=', 'statefile=', 'alarmfile=' ])
	except getopt.GetoptError:
   	   usage()
   	   sys.exit(1)

	vetofilename='/etc/glite-data-transfer-agents.disabled'
	statefile='/var/lib/fts-agent-alarm/fts-agent.state'
	alarmfile='/var/lib/fts-agent-alarm/fts-agent.alarm'

        try:
           os.mkdir('/var/lib/fts-agent-alarm')
        except OSError:
           pass
           # already exists

	for o,a in opts[0]:
	   if o == '--debug': debug=True
	   if o == '--vetofile':
	      vetofilename=a
	   if o == '--statefile':
	      statefile=a
	   if o == '--alarmfile':
	      alarmfile=a

	# Read veto list: agents in this list will be ignored by sensor
	if os.path.isfile(vetofilename):
	   vetofile = open(vetofilename,"r")
	   vetolist = vetofile.readlines()
	   vetofile.close()
	   vetolist = map(string.strip, vetolist)
	else:
	   if debug:
	      log_message('No veto file specified!')
	   vetolist=[]

	# Snapshot process names
	procs=os.listdir('/proc')
	procs=fnmatch.filter(procs, '[0-9]*')
	processtable={}
	for proc in procs:
	   fn='/proc/' + proc + '/cmdline'
	   try:
	      fil=open(fn, 'r')
	   except IOError:
	      continue
	   psname=fil.readline()
	   fil.close()
	   if psname is not None:
		psname = string.strip(psname)
		psname = string.translate(psname, string.maketrans(string.printable, string.printable), '\x00')
	   processtable[psname] = proc

	# Check lockfile
	if os.path.exists('/var/lock/subsys/fts-agent-check'):
	   # read it and get pid
	   lockfile=open('/var/lock/subsys/fts-agent-check', 'r')
	   lockpid=lockfile.readline()
	   lockpid=string.strip(lockpid)
	   lockfile.close()
	   # now check the pid existed a few microseconds ago
	   for i in procs:
	      if lockpid == i:
		 log_message('ERROR: lockfile exists and process exists.. not starting!')
		 sys.exit(5)
	   else:
		log_message('WARN: lockfile exists, but no process.. acquiring lock and running..')
	   os.remove('/var/lock/subsys/fts-agent-check')

	lockfile=open('/var/lock/subsys/fts-agent-check', 'w')
	lockfile.write(`os.getpid()` + '\n')
	lockfile.close()   

	badagents=[]

	agents = os.listdir('/opt/glite/etc/glite-data-transfer-agents.d')
	agents = fnmatch.filter(agents, 'glite-transfer-*agent*.properties.xml')
	for agent in agents:
	   agent = re.sub('\.properties.xml','', agent)
	   if agent in vetolist:
               if debug:
	          log_message('Ignoring: ' + agent)
	       continue
	   if agent in processtable.keys(): 
	       if debug:
		  log_message('Running: ' + agent + ' with PID ' + processtable[agent])
	   else:
	       log_message('ERROR: ' + agent)
	       badagents.append(agent)

	reporterror = []
	newstatedic = {}
	if badagents != []:
	   #if debug: log_message('Restarting ' + `len(badagents)` +' agents that are down...')
	   if statefile is None:
		if debug: log_message('Statefile not specified: no action taken...')
		os.remove('/var/lock/subsys/fts-agent-check')
		sys.exit(0)
	   statedic = {}
	   if os.path.isfile(statefile):
	      statef = open(statefile,"r")
	      statelist = statef.readlines()
	      statef.close()
	      statelist = map(string.strip, statelist)
	      for i in statelist:
		 ag, attempts = i.split(' ')
		 statedic[ag] = attempts
	   for i in badagents:
	      if i in statedic.keys():
		 num_attempts=string.atoi(statedic[i])+1
		 if num_attempts > 2:
		    reporterror.append(i)
		    newstatedic[i]=3
		    continue
		 log_message(i + ': is still down on pass ' + `num_attempts`) 
	      else:
		 num_attempts=1
		 #log_message(i + ': new attempt to restart')
	      #syslog.syslog('Attempting restart of down FTS agent <' + i + '>. Attempt number ' + `num_attempts` + '.')
	      #restartok = do_restart(i)
	      newstatedic[i]=num_attempts
	   
	# write the new state file
	statef = open(statefile,"w")
	for j in newstatedic.keys():
	   statef.write(j + " " + `newstatedic[j]` + '\n')
	statef.close()

	# now write the error report file; always move it first
	if os.path.isfile(alarmfile):
	    os.rename(alarmfile, alarmfile+'.old')
	if reporterror != []:
	    log_message('Writing ALARM report for ' + `len(reporterror)` + ' agents...')
	    alarm = open(alarmfile,"w")
	    for i in reporterror:
	       alarm.write(i + ' BAD \n')
	    alarm.close()

	# Delete lockfile
	os.remove('/var/lock/subsys/fts-agent-check')

if __name__ == "__main__":
    main()
