-- G. McCance May 2006

-- Make history tables

CREATE TABLE X_JOBIDS (
	JOB_ID  CHAR(36));

ALTER TABLE X_JOBIDS ADD (
	CONSTRAINT X_JOBIDS_PK PRIMARY KEY (JOB_ID));
	
CREATE TABLE X_FILEIDS (
	FILE_ID  NUMBER(38));

ALTER TABLE X_FILEIDS ADD (
	CONSTRAINT X_FILEIDS_PK PRIMARY KEY (FILE_ID));

CREATE TABLE T_HISTORY_LOG (
	STARTTIME   TIMESTAMP(6) WITH TIME ZONE,
	FINISHTIME  TIMESTAMP(6) WITH TIME ZONE,
	JOBS        NUMBER,
	FILES       NUMBER,
	TRANSFERS   NUMBER,
	ERRCODE     NUMBER,
	ERRMSG      VARCHAR2(2048));

CREATE TABLE T_JOB_HISTORY (
	JOB_ID               CHAR(36) NOT NULL,
	JOB_STATE            VARCHAR2(13) NOT NULL,
	JOB_PARAMS           VARCHAR2(255),
	SOURCE               VARCHAR2(255),
	DEST                 VARCHAR2(255),
	USER_DN              VARCHAR2(1024) NOT NULL,
	AGENT_DN             VARCHAR2(1024),
	USER_CRED            VARCHAR2(255),
	VOMS_CRED            BLOB,
	VO_NAME              VARCHAR2(50),
	CHANNEL_NAME         VARCHAR2(32),
	REASON               VARCHAR2(1024),
	PRIORITY             NUMBER(38),
	INTERNAL_JOB_PARAMS  VARCHAR2(255),
	SUBMIT_TIME          TIMESTAMP(6) WITH TIME ZONE,
	SUBMIT_HOST          VARCHAR2(100),
	MAX_TIME_IN_QUEUE    NUMBER(38));

CREATE INDEX JOB_HISTORY_JOB_ID    ON T_JOB_HISTORY(JOB_ID);
CREATE INDEX JOB_HISTORY_JOB_STATE    ON T_JOB_HISTORY(JOB_STATE);
CREATE INDEX JOB_HISTORY_CHANNEL_NAME ON T_JOB_HISTORY(CHANNEL_NAME);
CREATE INDEX JOB_HISTORY_VO_NAME      ON T_JOB_HISTORY(VO_NAME);
	
		
CREATE TABLE T_FILE_HISTORY (
	FILE_ID           NUMBER(38) NOT NULL,
	JOB_ID            CHAR(36) NOT NULL,
	FILE_STATE        VARCHAR2(29) NOT NULL,
	LOGICAL_NAME      VARCHAR2(1100),
	SOURCE_SURL       VARCHAR2(1100),
	DEST_SURL         VARCHAR2(1100),
	AGENT_DN          VARCHAR2(1024),
	REASON_CLASS      VARCHAR2(32),
	REASON            VARCHAR2(1024),
	NUM_FAILURES      NUMBER(38),
	FILESIZE          NUMBER(38),
	CHECKSUM          VARCHAR2(100),
	CURRENT_FAILURES  NUMBER(38),
	FINISH_TIME       TIMESTAMP(6) WITH TIME ZONE);
	
CREATE INDEX FILE_HISTORY_FILE_ID      ON T_FILE_HISTORY(FILE_ID); 
CREATE INDEX FILE_HISTORY_JOB_ID       ON T_FILE_HISTORY(JOB_ID);
CREATE INDEX T_FILE_HISTORY_FILE_STATE ON T_FILE_HISTORY(FILE_STATE);

	
CREATE TABLE T_TRANSFER_HISTORY (
	REQUEST_ID      VARCHAR2(255) NOT NULL,
	FILE_ID         NUMBER(38) NOT NULL,
	TRANSFER_ID     NUMBER(38) NOT NULL,
	TRANSFER_STATE  VARCHAR2(29) NOT NULL,
	SOURCE_TURL     VARCHAR2(1100),
	DEST_TURL       VARCHAR2(1100),
	DURATION        NUMBER(12,2),
	BYTES_WRITTEN   NUMBER(38),
	REASON_CLASS    VARCHAR2(32),
	REASON          VARCHAR2(1024),
	TRANSFER_TIME   TIMESTAMP(6) WITH TIME ZONE);

CREATE INDEX T_TRANSFER_HISTORY_T_ID              ON T_TRANSFER_HISTORY(TRANSFER_ID);
CREATE INDEX T_TRANSFER_HISTORY_REQUEST_ID        ON T_TRANSFER_HISTORY(REQUEST_ID);
CREATE INDEX T_TRANSFER_HISTORY_FILE_ID           ON T_TRANSFER_HISTORY(FILE_ID);
CREATE INDEX T_T_HISTORY_TRANSFER_STATE           ON T_TRANSFER_HISTORY(TRANSFER_STATE);
