-- G. McCance May 2006

CREATE OR REPLACE PACKAGE BODY fts_admin_pack
IS 

-- PRIVATE variables
e_no_constraint EXCEPTION;
PRAGMA EXCEPTION_INIT(e_no_constraint, -20101);
e_badschema EXCEPTION;
PRAGMA EXCEPTION_INIT(e_badschema, -20103);
e_badchannel EXCEPTION;
PRAGMA EXCEPTION_INIT(e_badchannel, -20102);
e_badagents EXCEPTION;
PRAGMA EXCEPTION_INIT(e_badagents, -20104);

    
-- PRIVATE methods

FUNCTION get_channel_job_constraint
RETURN USER_CONS_COLUMNS.CONSTRAINT_NAME%TYPE
IS
v_cons_name USER_CONS_COLUMNS.CONSTRAINT_NAME%TYPE;
BEGIN
   select constraint_name into v_cons_name from user_cons_columns where table_name='T_JOB' and column_name='CHANNEL_NAME';
   dbms_output.put_line('Identified T_JOB / T_CHANNEL referential constraint as: ' || v_cons_name);
   RETURN v_cons_name;
EXCEPTION
   WHEN NO_DATA_FOUND THEN
      dbms_output.put_line('ERROR! Cannot identify T_JOB / T_CHANNEL referential constraint!');
      RAISE_APPLICATION_ERROR(-20101, 'ERROR! Cannot identify T_JOB / T_CHANNEL referential constraint!');
   WHEN TOO_MANY_ROWS THEN
      dbms_output.put_line('ERROR! Cannot uniquely identify T_JOB / T_CHANNEL referential constraint!');
      RAISE_APPLICATION_ERROR(-20101, 'ERROR! Cannot uniquely identify T_JOB / T_CHANNEL referential constraint!');
END get_channel_job_constraint; 
          
      
PROCEDURE check_nothing_is_running
IS
v_agent_name T_AGENT.AGENT_NAME%TYPE;
BEGIN
  select AGENT_NAME into v_agent_name from t_agent where agent_state != 'Stopped' and rownum < 2;
  dbms_output.put_line('ERROR: There are FTA agent daemons running! This tool must be run with no agents running!');
  RAISE_APPLICATION_ERROR(-20104, 'ERROR: There are FTA agent daemons running! This tool must be run with no agents running!');
EXCEPTION
  WHEN TOO_MANY_ROWS THEN
     dbms_output.put_line('ERROR: There are FTA agent daemons running! This tool must be run with no agents running!');
     RAISE_APPLICATION_ERROR(-20104, 'ERROR: There are FTA agent daemons running! This tool must be run with no agents running!');   
  WHEN NO_DATA_FOUND THEN
     -- good
     null;
END check_nothing_is_running;
                 
  
PROCEDURE check_schema_version
IS
v_schema_vers VARCHAR(20);
BEGIN
   select MAJOR||'.'||MINOR||'.'||PATCH into v_schema_vers from T_SCHEMA_VERS;
   IF v_schema_vers != VALID_SCHEMA_VERSION THEN
      dbms_output.put_line('ERROR: This script only runs with schema version: '||VALID_SCHEMA_VERSION);
      dbms_output.put_line('       You have version: '||v_schema_vers);
      RAISE_APPLICATION_ERROR(-20103, 'This script only runs with schema version: '||VALID_SCHEMA_VERSION);
   END IF;
END check_schema_version;

  
-- PUBLIC method bodies

-- Change source site name on channel
PROCEDURE change_channel_source
   ( v_channel_name T_CHANNEL.CHANNEL_NAME%TYPE,
     v_channel_source T_CHANNEL.SOURCE_SITE%TYPE )
IS
BEGIN
   check_schema_version;
   UPDATE T_CHANNEL set SOURCE_SITE=v_channel_source WHERE CHANNEL_NAME = v_channel_name;
   COMMIT;
EXCEPTION
   WHEN OTHERS THEN
      dbms_output.put_line('Error updating channel name: '||v_channel_name);
      dbms_output.put_line('  Cannot update source site name');
      dbms_output.put_line('  SQLError code: '||SQLCODE);
      dbms_output.put_line('  SQLError message: '||SQLERRM);  
END change_channel_source;
   
   
-- Change destination site name on channel    
PROCEDURE change_channel_dest
   ( v_channel_name T_CHANNEL.CHANNEL_NAME%TYPE,
     v_channel_dest T_CHANNEL.DEST_SITE%TYPE )
IS
BEGIN
   check_schema_version;
   UPDATE T_CHANNEL set DEST_SITE=v_channel_dest WHERE CHANNEL_NAME = v_channel_name;
   COMMIT;
EXCEPTION
   WHEN OTHERS THEN
      dbms_output.put_line('Error updating channel name: '||v_channel_name);
      dbms_output.put_line('  Cannot update destination site name');
      dbms_output.put_line('  SQLError code: '||SQLCODE);
      dbms_output.put_line('  SQLError message: '||SQLERRM);      
END change_channel_dest;
   
      
-- Rename channel and the foreign key references to it
PROCEDURE rename_channel
   ( v_channel_name_old T_CHANNEL.CHANNEL_NAME%TYPE,
     v_channel_name_new T_CHANNEL.CHANNEL_NAME%TYPE )
IS
v_constraint USER_CONS_COLUMNS.CONSTRAINT_NAME%TYPE;
BEGIN 
   check_schema_version;
   check_nothing_is_running;
   dbms_output.put_line('Dropping database constraints to update channel name...');
   v_constraint := get_channel_job_constraint();
   execute immediate('ALTER TABLE T_JOB DISABLE CONSTRAINT '||v_constraint); 
   
   dbms_output.put_line('  Changing channel '||v_channel_name_old||' to '||v_channel_name_new||' in T_CHANNEL...');  
   UPDATE T_CHANNEL set CHANNEL_NAME=v_channel_name_new WHERE CHANNEL_NAME = v_channel_name_old;
   IF SQL%ROWCOUNT = 0 THEN
      dbms_output.put_line('  ERROR: Channel name <'||v_channel_name_new||'> not found!');
      RAISE_APPLICATION_ERROR(-20102, 'ERROR: Old channel name <'||v_channel_name_old||'> not found!');
   END IF;
   
   dbms_output.put_line('  Updating foreign key references for channel in T_JOB...');
   UPDATE T_JOB set CHANNEL_NAME=v_channel_name_new WHERE CHANNEL_NAME = v_channel_name_old;     
   dbms_output.put_line('  Updating foreign key references for channel in T_CHANNEL_ACL...');
   UPDATE T_CHANNEL_ACL set CHANNEL_NAME=v_channel_name_new WHERE CHANNEL_NAME = v_channel_name_old;    
   dbms_output.put_line('  Updating foreign key references for channel in T_CHANNEL_VO_SHARE...');
   UPDATE T_CHANNEL_VO_SHARE set CHANNEL_NAME=v_channel_name_new WHERE CHANNEL_NAME = v_channel_name_old;
   dbms_output.put_line('  Updating foreign key references for channel in T_TRANSFER_CUMULATIVE_STATS...');
   UPDATE T_TRANSFER_CUMULATIVE_STATS set CHANNEL_NAME=v_channel_name_new WHERE CHANNEL_NAME = v_channel_name_old;  
   
   dbms_output.put_line('Re-enabling database constraints...');
   execute immediate('ALTER TABLE T_JOB ENABLE CONSTRAINT '||v_constraint);      
       
EXCEPTION
   WHEN e_no_constraint or e_badschema or e_badagents THEN
      dbms_output.put_line('Error changing channel name: '||v_channel_name_old);
      dbms_output.put_line('  Cannot change the channel name to: '||v_channel_name_new);
      dbms_output.put_line('  SQLError code: '||SQLCODE);
      dbms_output.put_line('  SQLError message: '||SQLERRM);
   WHEN OTHERS THEN
      dbms_output.put_line('Error changing channel name: '||v_channel_name_old);
      dbms_output.put_line('  Cannot change the channel name to: '||v_channel_name_new);
      dbms_output.put_line('  SQLError code: '||SQLCODE);
      dbms_output.put_line('  SQLError message: '||SQLERRM); 
      
      dbms_output.put_line('Re-enabling database constraints...');
      execute immediate('ALTER TABLE T_JOB ENABLE CONSTRAINT '||v_constraint);      
END rename_channel;    
     
END fts_admin_pack;
/
