-- G. McCance May 2006

create or replace package body fts_history
as

-- PRIVATE methods
procedure clearvars
as
begin
  v_starttime := NULL;
  v_endtime := NULL;
  v_nojobs := NULL;
  v_nofiles := NULL;
  v_notransfers := NULL;  
end clearvars;

procedure cleartables
as
begin
  execute immediate('truncate table X_JOBIDS');
  execute immediate('truncate table X_FILEIDS');
end cleartables;

procedure get_jobids
   (v_days binary_integer,
    v_jobs binary_integer)
as
begin
  execute immediate('insert into x_jobids 
                       select distinct(job_id) from t_job where 
                              t_job.job_state IN (''Done'', ''Failed'', ''Canceled'', ''Finished'')
                              and t_job.submit_time < ( systimestamp - interval '''||v_days||''' DAY ) 
                              and rownum <= '||v_jobs);
  select count(job_id) into v_nojobs FROM X_JOBIDS;  
end get_jobids;
 
procedure get_fileids
as
begin
  execute immediate('insert into x_fileids 
                       select distinct(file_id) from t_file where job_id IN (select * from x_jobids)');
  select count(file_id) into v_nofiles FROM X_FILEIDS;
  select count(transfer_id) into v_notransfers FROM T_TRANSFER T WHERE T.file_id in (select * from x_fileids); 
end get_fileids;

procedure copy_transfers
as
begin
  execute immediate('insert into t_transfer_history select * from t_transfer where file_id in (select * from x_fileids)'); 
end copy_transfers;
  
procedure copy_files
as
begin
  execute immediate('insert into t_file_history select * from t_file where file_id in (select * from x_fileids)'); 
end copy_files;
          
procedure copy_jobs
as
begin
  execute immediate('insert into t_job_history select * from t_job where job_id in (select * from x_jobids)'); 
end copy_jobs;
      
      
procedure delete_transfers
as
begin
  execute immediate('delete from t_transfer where file_id in (select * from x_fileids)'); 
end delete_transfers;  

procedure delete_files
as
begin
  execute immediate('delete from t_file where file_id in (select * from x_fileids)'); 
end delete_files;

procedure delete_jobs
as
begin
  execute immediate('delete from t_job where job_id in (select * from x_jobids)'); 
end delete_jobs;

procedure logitsuccess
as
begin
  execute immediate('insert into t_history_log (STARTTIME, FINISHTIME, JOBS, FILES, TRANSFERS) VALUES (:1, :2, :3, :4, :5)')
      using v_starttime, v_endtime, v_nojobs, v_nofiles, v_notransfers;  
end logitsuccess;

procedure logitfailure
   ( errcode number, errmessage varchar)
as
begin
  execute immediate('insert into t_history_log (STARTTIME, FINISHTIME, JOBS, FILES, TRANSFERS, ERRCODE, ERRMSG) 
                                                VALUES (:1, :2, :3, :4, :5, :6, SUBSTR(:7, 1, 2048))')
      using v_starttime, v_endtime, v_nojobs, v_nofiles, v_notransfers, errcode, errmessage;  
end logitfailure;

-- PUBLIC methods
procedure movedata
   (v_days binary_integer,
    v_jobs binary_integer)
as
begin
  clearvars;
  select SYSTIMESTAMP into v_starttime from DUAL;
  cleartables;
  get_jobids(v_days, v_jobs);
  get_fileids;
  commit;
  copy_transfers;
  copy_files;
  copy_jobs;  
  delete_transfers;
  delete_files;  
  delete_jobs;
  commit;
  select SYSTIMESTAMP into v_endtime from DUAL;
  logitsuccess;
EXCEPTION
   WHEN OTHERS THEN
       rollback;
       select SYSTIMESTAMP into v_endtime from DUAL;
       logitfailure(SQLCODE, SQLERRM);       
end movedata;
  
procedure submit_job
   (v_minutes binary_integer)
as
njob USER_JOBS.JOB%TYPE;
begin
   dbms_job.submit(njob, 'begin fts_history.movedata; end;', sysdate, 'SYSDATE + '||to_char(v_minutes)||'/1440'); 
end submit_job;

procedure stop_job
as
v_jobid USER_JOBS.JOB%TYPE;
begin
   select JOB into v_jobid from user_jobs where what like '%fts_history.movedata%';
   dbms_job.remove(v_jobid);
end stop_job;

end fts_history;
/
