-- G. McCance May 2006

create or replace package body fts_stats
IS
                      
-- private variables    
  
-- private methods
 
procedure log_error
  (v_file_id in number, v_code number, v_mess in varchar2)
AS
PRAGMA AUTONOMOUS_TRANSACTION;
BEGIN
  execute immediate 'insert into T_FILESTATS_LOG_ERR(LOGDATE, file_id, err_code, err_msg) values(SYSTIMESTAMP, :1, :2, SUBSTR(:3, 1, 2048))' 
    using v_file_id, v_code, v_mess;
  COMMIT;
END log_error;
          
-- public methods

procedure log_transfer
   ( v_file_id T_FILE.FILE_ID%TYPE,
     v_filesize T_FILE.FILESIZE%TYPE,
     v_num_failures T_FILE.NUM_FAILURES%TYPE,
     v_finish_time T_FILE.FINISH_TIME%TYPE,
     v_job_id T_FILE.JOB_ID%TYPE )
AS
BEGIN

execute immediate ('INSERT INTO T_FILESTATS_LOG (FID, LOGDATE, JOB_ID, FILE_ID, 
                                                 USER_DN, VO_NAME, CHANNEL_NAME,
                                                 SOURCE_SITE, DEST_SITE, FILESIZE,
                                                 NUM_FAILURES, FINISH_TIME) 
                     select FILESTATS_LOG_SEQ.NEXTVAL, SYSTIMESTAMP, J.job_id, :1, J.user_dn, J.vo_name, J.channel_name, C.source_site, C.dest_site,
                     :2, nvl(:3,0) as num_failures, :4
                     from t_job J, t_channel C where J.job_id = :5
                     and J.channel_name = C.channel_name') 
                     using v_file_id, v_filesize, v_num_failures, v_finish_time, v_job_id;
EXCEPTION
   WHEN OTHERS THEN
       log_error(v_file_id, SQLCODE, SQLERRM); 
END log_transfer;
                              
procedure make_logstats_table
AS
BEGIN
  execute immediate ('CREATE TABLE T_FILESTATS_LOG (
    FID           NUMBER,
    LOGDATE       TIMESTAMP(6) WITH TIME ZONE DEFAULT SYSTIMESTAMP,
	JOB_ID        CHAR(36) NOT NULL,
	FILE_ID       NUMBER(38) NOT NULL,
	USER_DN       VARCHAR2(1024) NOT NULL,
	VO_NAME       VARCHAR2(50),
	CHANNEL_NAME  VARCHAR2(32),
	SOURCE_SITE   VARCHAR2(100) NOT NULL,
	DEST_SITE     VARCHAR2(100) NOT NULL,
	FILESIZE      NUMBER(38),
	NUM_FAILURES  NUMBER,
	FINISH_TIME   TIMESTAMP(6) WITH TIME ZONE)');
  execute immediate ('CREATE INDEX FILESTATS_LOGDATE ON T_FILESTATS_LOG (LOGDATE)');
  execute immediate ('ALTER TABLE T_FILESTATS_LOG ADD ( CONSTRAINT FILESTATS_ID_PK PRIMARY KEY (FID) )');
  execute immediate ('CREATE TABLE T_FILESTATS_LOG_ERR (
        LOGDATE       TIMESTAMP(6) WITH TIME ZONE DEFAULT SYSTIMESTAMP,
    	FILE_ID       NUMBER(38) NOT NULL,
    	ERR_CODE      NUMBER NULL,
    	ERR_MSG       VARCHAR(2048))');
  execute immediate ('CREATE SEQUENCE FILESTATS_LOG_SEQ START WITH 1');
END make_logstats_table;

procedure prune_tables
AS
BEGIN
  null;
  --DELETE FROM  T_FILESTATS_LOG where LOGDATE < SYSTIMESTAMP - INTERVAL '24' HOUR';
  --DELETE FROM  T_FILESTATS_LOG_ERR where LOGDATE < SYSTIMESTAMP - INTERVAL '24' HOUR';  
END prune_tables;

procedure add_file_trigger
AS
v_trigger_state T_FILE.FILE_STATE%TYPE := 'Done';
BEGIN
  execute immediate('CREATE TRIGGER t_file_finalstate_trigger 
                     AFTER UPDATE OF FILE_STATE ON T_FILE
                     FOR EACH ROW
                     WHEN (NEW.FILE_STATE = ''Done'')
                     BEGIN
                       fts_stats.log_transfer(:NEW.FILE_ID, :NEW.FILESIZE, :NEW.NUM_FAILURES, :NEW.FINISH_TIME, :NEW.JOB_ID);
                     END;');
END;
  
procedure drop_file_trigger
AS
BEGIN
  execute immediate('DROP TRIGGER t_file_finalstate_trigger');
END;

END;
/
