-- G. McCance May 2006

create or replace package body fts_writer_account
is
   
-- PRIVATE methods
                 
procedure check_not_account
  (username in varchar)
as
begin
  if user = username THEN
     dbms_output.put_line('Both the other account and the account you are running from are the same!');
     dbms_output.put_line('You are running this from the wrong account!');
     RAISE_APPLICATION_ERROR(-20101, 'You are running this from the wrong account!'); 
  end if;    
end check_not_account;


-- PUBLIC methods   

procedure make_grants
  (writer_account in varchar)
as
 writer_account_u varchar(255);
begin
  dbms_output.put_line('Making grants...');
  writer_account_u := UPPER(writer_account);
  check_not_account(writer_account_u);
  FOR tab in ( select table_name from user_tables where table_name like 'T@_%' ESCAPE '@' ) LOOP
     dbms_output.put_line('Making grant to user '||writer_account_u||' for table '||tab.table_name);
     execute immediate('grant SELECT, UPDATE, INSERT, DELETE ON '||user||'.'||tab.table_name||' TO '||writer_account_u); 
  END LOOP;
  dbms_output.put_line('Making grant to user '||writer_account_u||' for package FTS_WRITER_ACCOUNT');
  execute immediate('GRANT execute on FTS_WRITER_ACCOUNT to '||writer_account_u);
end make_grants;

procedure make_synonyms
  (owner_account in varchar)
as
owner_account_u varchar(255);
synname varchar(255);
begin
  owner_account_u := UPPER(owner_account);
  check_not_account(owner_account_u);
  FOR tab in ( select distinct(table_name) from user_tab_privs where GRANTOR=owner_account_u 
               and GRANTEE=user and table_name like 'T@_%' ESCAPE '@' ) LOOP
     synname := tab.table_name;
     dbms_output.put_line('Making synonym for table '||owner_account_u||'.'||tab.table_name||
                          ' from user '||owner_account_u || ' as synonym '||synname);
     execute immediate('create synonym '||synname||' for '||owner_account_u||'.'||tab.table_name);
  END LOOP;
end make_synonyms;


end fts_writer_account;
/
